<?php

namespace Drupal\iconset\Plugin\Deriver;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Create derivative definitions for icon handlers which use directory assets.
 */
class IconDirectoryDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Create icon handlers for searching.
   *
   * @param string $base_plugin_id
   *   The plugin ID of the deriver definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct($base_plugin_id, ModuleHandlerInterface $module_handler) {
    $this->basePluginId = $base_plugin_id;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $base_plugin_id,
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $handlerTypes = [];
    $handlerTypes['svg'] = [
      'provider' => 'iconset',
      'libraries' => ['iconset/svg-icons'],
    ];
    $handlerTypes['image'] = [
      'provider' => 'iconset',
      'libraries' => ['iconset/svg-icons'],
    ];

    // Allow modules add other plugin types that can find icons directories.
    $handlerTypes += $this->moduleHandler->invokeAll('iconset_icon_directory_handler_type');
    $this->moduleHandler->alter('iconset_icon_directory_handler', $handlerTypes);

    // Iterate the icon directory handlers.
    foreach ($handlerTypes as $pluginId => $info) {
      $this->derivatives[$pluginId] = [
        'label' => $base_plugin_definition['label'],
        'help' => $base_plugin_definition['help'],
        'class' => $base_plugin_definition['class'],
      ] + $info;
    }

    return $this->derivatives;
  }

}
